@@ -6,10 +6,12 @@ use flexi_logger::{
 };
 use glob::glob;
 use log::*;
-use std::fs::File;
-use std::io::{BufRead, BufReader};
-use std::ops::Add;
-use std::thread::JoinHandle;
+use std::{
+    fs::File,
+    io::{BufRead, BufReader},
+    ops::Add,
+    thread::JoinHandle,
+};
 
 const NO_OF_THREADS: usize = 5;
 const NO_OF_LOGLINES_PER_THREAD: usize = 20_000;
@@ -20,48 +22,44 @@ fn multi_threaded() {
     // we use a special log line format that starts with a special string so that it is easier to
     // verify that all log lines are written correctly
 
-    let start = test_utils::now_local();
     let directory = test_utils::dir();
-    let logger = Logger::try_with_str("debug")
-        .unwrap()
-        .log_to_file(
-            FileSpec::default()
-                .basename("test_mtn")
-                .directory(&directory),
-        )
-        .write_mode(WriteMode::BufferAndFlush)
-        .format(test_format)
-        .duplicate_to_stderr(Duplicate::Info)
-        .rotate(
-            Criterion::Size(ROTATE_OVER_SIZE),
-            Naming::Numbers,
-            Cleanup::Never,
-        )
-        .start()
-        .unwrap_or_else(|e| panic!("Logger initialization failed with {}", e));
-    info!(
+    {
+        let _stopwatch = test_utils::Stopwatch::default();
+        let logger = Logger::try_with_str("debug")
+            .unwrap()
+            .log_to_file(
+                FileSpec::default()
+                    .basename("test_mtn")
+                    .directory(&directory),
+            )
+            .write_mode(WriteMode::BufferAndFlush)
+            .format(test_format)
+            .duplicate_to_stderr(Duplicate::Info)
+            .rotate(
+                Criterion::Size(ROTATE_OVER_SIZE),
+                Naming::Numbers,
+                Cleanup::Never,
+            )
+            .start()
+            .unwrap_or_else(|e| panic!("Logger initialization failed with {}", e));
+        info!(
         "create a huge number of log lines with a considerable number of threads, verify the log"
     );
 
-    let mut logger2 = logger.clone();
-    let worker_handles = start_worker_threads(NO_OF_THREADS);
-    let new_spec = LogSpecification::parse("trace").unwrap();
-    std::thread::Builder::new()
-        .spawn(move || {
-            std::thread::sleep(std::time::Duration::from_millis(500));
-            logger2.set_new_spec(new_spec);
-            0
-        })
-        .unwrap();
+        #[allow(clippy::redundant_clone)] // clippy ignores the Drop implementation :-(
+        let mut logger2 = logger.clone();
+        let worker_handles = start_worker_threads(NO_OF_THREADS);
+        let new_spec = LogSpecification::parse("trace").unwrap();
+        std::thread::Builder::new()
+            .spawn(move || {
+                std::thread::sleep(std::time::Duration::from_millis(500));
+                logger2.set_new_spec(new_spec);
+                0
+            })
+            .unwrap();
 
-    wait_for_workers_to_close(worker_handles);
-
-    let delta = (test_utils::now_local() - start).whole_milliseconds();
-    debug!(
-        "Task executed with {} threads in {} ms.",
-        NO_OF_THREADS, delta
-    );
-    logger.shutdown();
+        wait_for_workers_to_close(worker_handles);
+    }
     verify_logs(&directory.display().to_string());
 }
 
@@ -151,10 +149,10 @@ fn verify_logs(directory: &str) {
     }
     assert_eq!(
         line_count,
-        NO_OF_THREADS * NO_OF_LOGLINES_PER_THREAD + 3 + NO_OF_THREADS
+        NO_OF_THREADS * NO_OF_LOGLINES_PER_THREAD + NO_OF_THREADS + 2
     );
-    info!(
-        "Wrote {} log lines from {} threads into {} files",
+    println!(
+        "Found {} log lines from {} threads in {} files",
         line_count, NO_OF_THREADS, no_of_log_files
     );
 }
